
'''
 MAGIC  
 {M}y {A}utomated {G}rain-s{I}ze {C}ode'
 Python function to compute estimates of mean grain size (following Buscombe et al, 2010) 
 from an image of sediment where grains are clearly resolved
 DOES NOT REQUIRE CALIBRATION

 INPUTS:
 imagefile     a filename of an image which must be on the path (for example, 047080139.tiff) without quotes or apostrophes
 Optional input:
 res           spatial resolution of camera in mm/px (defaults to 1 - you can multiply by mm/pixel afterwards)
 dofilt        if 1, apply filter to remove large wavelengths of light which can affect results (defaults to 1)

 OUTPUTS:
 text file which contains:
 s1,s2,s3:         three measures of the mean of intermediate diameters. 
 MeanGrainSize:    Usually, point counts correspond most closely to the minimum of these three, which is what this is

 References
 1) Buscombe D., Rubin, D.M., and Warrick, J.A. (2010) Universal Approximation to 
 Grain Size from Images of Non-Cohesive Unlithified Sediment. 
 Journal of Geophysical Research - Earth Surface 115,F02015

 2) W. Gander, G. H. Golub, R. Strebel (1994) Least-Squares Fitting of Circles and Ellipses, 
 BIT Numerical Mathematics,  Springer. 

 Examples:
 1) use all defaults
 python magic.py 047080139.tiff

 2) use own image resolution (outut in real length units rather than pixels)
 python magic.py 047080139.tiff 0.01

 3) use own image resolution and turn filtering off (saves time)
 python magic.py 047080139.tiff 0.01 0 

  Author:  Daniel Buscombe
           School of Marine Science & Engineering
           University of Plymouth, Drake Circus, Plymouth, Devon PL48AA, UK
           daniel.buscombe@plymouth.ac.uk
  Version: Beta        Revision: 1 May, 2010
  First Revision 1 May 2010    
  This revision 31 May 2011                         
  tested under Ubuntu 9.04 and 10.04 using Python 2.6.2
  tested under Windows XP using Python 2.6.6

 The following packages need to be installed (you may need administrative privileges)
 1. Python - already included in most Mac and Linux operating systems
 WINDOWS:
 download: python-2.6.6.msi (Windows x86 MSI Installer, or Windows x86-64 MSI Installer if you have a 64-bit computer) 
 install
 (note: install for 'all users' even if only you will be using it)

 2. PIL (Python Imaging Library)
 homepage and download: http://www.pythonware.com/products/pil/
 MAC/LINUX
 download, then issue the following commands in the terminal
        $ tar xvf Imaging-1.1.6.tar
        $ cd Imaging-1.1.6
        $ python setup.py install
 WINDOWS
 http://effbot.org/downloads/PIL-1.1.7.win32-py2.6.exe
 (If you get an error saying that your computer is missing msvcr71.dll, then download from here:
http://www.dll-files.com/dllindex/dll-files.shtml?msvcr71
then unzip the file and place the file msvcr71.dll in C:\WINDOWS\system32)

 3. matplotlib (plotting library)
 homepage: http://matplotlib.sourceforge.net/
 download: http://sourceforge.net/projects/matplotlib/

 4. NumPy (numerical python)
 homepage: http://numpy.scipy.org/
 download: http://sourceforge.net/projects/numpy/files/

 5. SciPy (scientific library for python)
 download: http://sourceforge.net/projects/scipy/files/

 (Macintosh users: dmg self-installing packages are available for matplotlib, numpy and scipy
 PIL has to be installed from source
 Ubuntu users: self-installing packages available through synaptic package manager)

'''

import sys #for input arguments

# declare libraries to use
import Image
from pylab import*
import cmath
from numpy import*
#from matplotlib.numerix import where, nonzero
#from matplotlib.numerix.mlab import rand
from math import atan2
from scipy.signal import fftconvolve # for 2d polynomial fitting
from scipy.misc import toimage  # for image rescaling

# begin sub-functions (where not written by myself, the author has been cited)

#Ellipse fitting ported from the matlab file fitellipse.m by Richard Brown.
#The porting was done by Yotam Gingold <yotam (strudel) slackworks.com>.
def fitellipse( x, opt = 'nonlinear', **kwargs ):
    '''
    FITELLIPSE   least squares fit of ellipse to 2D data
    
       [Z, A, B, ALPHA] = FITELLIPSE(X)
           Fit an ellipse to the 2D points in the 2xN array X.
           Fitting is performed by nonlinear least squares, optimising the
           squared sum of orthogonal distances from the points to the fitted
           ellipse. The initial guess is calculated by a linear least squares
           routine, by default using the Bookstein constraint (see below)
    '''

    x = asarray( x )
    
    ## Parse inputs
    # ...
    ## Default parameters
    kwargs[ 'fNonlinear' ] = opt is not 'linear'
    kwargs.setdefault( 'constraint', 'bookstein' )
    kwargs.setdefault( 'maxits', 200 )
    kwargs.setdefault( 'tol', 1e-5 )
    if x.shape[1] == 2:
        x = x.T
    if x.shape[1] < 6:
        raise RuntimeError, 'fitellipse:InsufficientPoints At least 6 points required to compute fit'
    
    ## Constraints are Euclidean-invariant, so improve conditioning by removing
    ## centroid
    centroid = mean(x, 1)
    x        = x - centroid.reshape((2,1))
    
    ## Obtain a linear estimate
    if kwargs['constraint'] == 'bookstein':
        ## Bookstein constraint : lambda_1^2 + lambda_2^2 = 1
        z, a, b, alpha = fitbookstein(x)
    
    elif kwargs['constraint'] == 'trace':
        ## 'trace' constraint, lambda1 + lambda2 = trace(A) = 1
        z, a, b, alpha = fitggk(x)
    
    ## Minimise geometric error using nonlinear least squares if required
    if kwargs['fNonlinear']:
        ## Initial conditions
        z0     = z
        a0     = a
        b0     = b
        alpha0 = alpha
        
        ## Apply the fit
        z, a, b, alpha, fConverged = fitnonlinear(x, z0, a0, b0, alpha0, **kwargs)
        
        ## Return linear estimate if GN doesn't converge
        if not fConverged:
            print 'fitellipse:FailureToConverge', 'Gauss-Newton did not converge, returning linear estimate'
            z = z0
            a = a0
            b = b0
            alpha = alpha0
    
    ## Add the centroid back on
    z = z + centroid
    
    return z, a, b, alpha

def fitbookstein(x):
    '''
    function [z, a, b, alpha] = fitbookstein(x)
    FITBOOKSTEIN   Linear ellipse fit using bookstein constraint
       lambda_1^2 + lambda_2^2 = 1, where lambda_i are the eigenvalues of A
    '''
    
    ## Convenience variables
    m  = x.shape[1]
    x1 = x[0, :].reshape((1,m)).T
    x2 = x[1, :].reshape((1,m)).T
    
    ## Define the coefficient matrix B, such that we solve the system
    ## B *[v; w] = 0, with the constraint norm(w) == 1
    B = hstack([ x1, x2, ones((m, 1)), power( x1, 2 ), multiply( sqrt(2) * x1, x2 ), power( x2, 2 ) ])
    
    ## To enforce the constraint, we need to take the QR decomposition
    Q, R = linalg.qr(B)
    
    ## Decompose R into blocks
    R11 = R[0:3, 0:3]
    R12 = R[0:3, 3:6]
    R22 = R[3:6, 3:6]
    
    ## Solve R22 * w = 0 subject to norm(w) == 1
    U, S, V = linalg.svd(R22)
    V = V.T
    w = V[:, 2]
    
    ## Solve for the remaining variables
    v = dot( linalg.solve( -R11, R12 ), w )
    
    ## Fill in the quadratic form
    A        = zeros((2,2))
    A.ravel()[0]     = w.ravel()[0]
    A.ravel()[1:3] = 1 / sqrt(2) * w.ravel()[1]
    A.ravel()[3]     = w.ravel()[2]
    bv       = v[0:2]
    c        = v[2]
    
    ## Find the parameters
    z, a, b, alpha = conic2parametric(A, bv, c)
    
    return z, a, b, alpha

def fitggk(x):
    '''
    function [z, a, b, alpha] = fitggk(x)
     Linear least squares with the Euclidean-invariant constraint Trace(A) = 1
    '''
    
    ## Convenience variables
    m  = x.shape[1]
    x1 = x[0, :].reshape((1,m)).T
    x2 = x[1, :].reshape((1,m)).T
    
    ## Coefficient matrix
    B = hstack([ multiply( 2 * x1, x2 ), power( x2, 2 ) - power( x1, 2 ), x1, x2, ones((m, 1)) ])
    
    v = linalg.lstsq( B, -power( x1, 2 ) )[0].ravel()
    
    ## For clarity, fill in the quadratic form variables
    A        = zeros((2,2))
    A[0,0]   = 1 - v[1]
    A.ravel()[1:3] = v[0]
    A[1,1]   = v[1]
    bv       = v[2:4]
    c        = v[4]
    
    ## find parameters
    z, a, b, alpha = conic2parametric(A, bv, c)
    
    return z, a, b, alpha


def fitnonlinear(x, z0, a0, b0, alpha0, **params):
    '''
    function [z, a, b, alpha, fConverged] = fitnonlinear(x, z0, a0, b0, alpha0, params)
     Gauss-Newton least squares ellipse fit minimising geometric distance 
    '''
    
    ## Get initial rotation matrix
    Q0 = array( [[ cos(alpha0), -sin(alpha0) ], [ sin(alpha0), cos(alpha0) ]] )
    m = x.shape[1]
    
    ## Get initial phase estimates
    phi0 = angle( dot( dot( array([1, 1j]), Q0.T ), x - z0.reshape((2,1)) ) ).T
    u = hstack( [ phi0, alpha0, a0, b0, z0 ] ).T


    def sys(u):
        '''
        function [f, J] = sys(u)
         SYS : Define the system of nonlinear equations and Jacobian. Nested
         function accesses X (but changeth it not)
         from the FITELLIPSE workspace
        '''
        
        ## Tolerance for whether it is a circle
        circTol = 1e-5
        
        ## Unpack parameters from u
        phi   = u[:-5]
        alpha = u[-5]
        a     = u[-4]
        b     = u[-3]
        z     = u[-2:]
        
        ## If it is a circle, the Jacobian will be singular, and the
        ## Gauss-Newton step won't work. 
        ##TODO: This can be fixed by switching to a Levenberg-Marquardt
        ##solver
        if abs(a - b) / (a + b) < circTol:
            print 'fitellipse:CircleFound', 'Ellipse is near-circular - nonlinear fit may not succeed'
        
        ## Convenience trig variables
        c = cos(phi)
        s = sin(phi)
        ca = cos(alpha)
        sa = sin(alpha)
        
        ## Rotation matrices
        Q    = array( [[ca, -sa],[sa, ca]] )
        Qdot = array( [[-sa, -ca],[ca, -sa]] )
        
        ## Preallocate function and Jacobian variables
        f = zeros(2 * m)
        J = zeros((2 * m, m + 5))
        for i in range( m ):
            rows = range( (2*i), (2*i)+2 )
            ## Equation system - vector difference between point on ellipse
            ## and data point
            f[ rows ] = x[:, i] - z - dot( Q, array([ a * cos(phi[i]), b * sin(phi[i]) ]) )
            
            ## Jacobian
            J[ rows, i ] = dot( -Q, array([ -a * s[i], b * c[i] ]) )
            J[ rows, -5: ] = \
                hstack([ ascol( dot( -Qdot, array([ a * c[i], b * s[i] ]) ) ), ascol( dot( -Q, array([ c[i], 0 ]) ) ), ascol( dot( -Q, array([ 0, s[i] ]) ) ), array([[-1, 0],[0, -1]]) ])
        
        return f,J
    
    
    ## Iterate using Gauss Newton
    fConverged = False
    for nIts in range( params['maxits'] ):
        ## Find the function and Jacobian
        f, J = sys(u)
        
        ## Solve for the step and update u
        #h = linalg.solve( -J, f )
        h = linalg.lstsq( -J, f )[0]
        u = u + h
        
        ## Check for convergence
        delta = linalg.norm(h, inf) / linalg.norm(u, inf)
        if delta < params['tol']:
            fConverged = True
            break
    
    alpha = u[-5]
    a     = u[-4]
    b     = u[-3]
    z     = u[-2:]
    
    return z, a, b, alpha, fConverged


def conic2parametric(A, bv, c):
    '''
    function [z, a, b, alpha] = conic2parametric(A, bv, c)
    '''
    ## Diagonalise A - find Q, D such at A = Q' * D * Q
    D, Q = linalg.eig(A)
    Q = Q.T
    
    ## If the determinant < 0, it's not an ellipse
    if prod(D) <= 0:
        raise RuntimeError, 'fitellipse:NotEllipse Linear fit did not produce an ellipse'
    
    ## We have b_h' = 2 * t' * A + b'
    t = -0.5 * linalg.solve(A, bv)
    
    c_h = dot( dot( t.T, A ), t ) + dot( bv.T, t ) + c
    
    z = t
    a = sqrt(-c_h / D[0])
    b = sqrt(-c_h / D[1])
    alpha = atan2(Q[0,1], Q[0,0])
    
    return z, a, b, alpha

def sgolay2d ( z, window_size, order, derivative=None):
    """
    from http://www.scipy.org/Cookbook/SavitzkyGolay
    """
    # number of terms in the polynomial expression
    n_terms = ( order + 1 ) * ( order + 2)  / 2.0

    if  window_size % 2 == 0:
        raise ValueError('window_size must be odd')

    if window_size**2 < n_terms:
        raise ValueError('order is too high for the window size')

    half_size = window_size // 2

    # exponents of the polynomial. 
    # p(x,y) = a0 + a1*x + a2*y + a3*x^2 + a4*y^2 + a5*x*y + ... 
    # this line gives a list of two item tuple. Each tuple contains 
    # the exponents of the k-th term. First element of tuple is for x
    # second element for y.
    # Ex. exps = [(0,0), (1,0), (0,1), (2,0), (1,1), (0,2), ...]
    exps = [ (k-n, n) for k in range(order+1) for n in range(k+1) ]

    # coordinates of points
    ind = np.arange(-half_size, half_size+1, dtype=np.float64)
    dx = np.repeat( ind, window_size )
    dy = np.tile( ind, [window_size, 1]).reshape(window_size**2, )

    # build matrix of system of equation
    A = np.empty( (window_size**2, len(exps)) )
    for i, exp in enumerate( exps ):
        A[:,i] = (dx**exp[0]) * (dy**exp[1])

    # pad input array with appropriate values at the four borders
    new_shape = z.shape[0] + 2*half_size, z.shape[1] + 2*half_size
    Z = np.zeros( (new_shape) )
    # top band
    band = z[0, :]
    Z[:half_size, half_size:-half_size] =  band -  np.abs( np.flipud( z[1:half_size+1, :] ) - band )
    # bottom band
    band = z[-1, :]
    Z[-half_size:, half_size:-half_size] = band  + np.abs( np.flipud( z[-half_size-1:-1, :] )  -band )
    # left band
    band = np.tile( z[:,0].reshape(-1,1), [1,half_size])
    Z[half_size:-half_size, :half_size] = band - np.abs( np.fliplr( z[:, 1:half_size+1] ) - band )
    # right band
    band = np.tile( z[:,-1].reshape(-1,1), [1,half_size] )
    Z[half_size:-half_size, -half_size:] =  band + np.abs( np.fliplr( z[:, -half_size-1:-1] ) - band )
    # central band
    Z[half_size:-half_size, half_size:-half_size] = z

    # top left corner
    band = z[0,0]
    Z[:half_size,:half_size] = band - np.abs( np.flipud(np.fliplr(z[1:half_size+1,1:half_size+1]) ) - band )
    # bottom right corner
    band = z[-1,-1]
    Z[-half_size:,-half_size:] = band + np.abs( np.flipud(np.fliplr(z[-half_size-1:-1,-half_size-1:-1]) ) - band )

    # top right corner
    band = Z[half_size,-half_size:]
    Z[:half_size,-half_size:] = band - np.abs( np.flipud(Z[half_size+1:2*half_size+1,-half_size:]) - band )
    # bottom left corner
    band = Z[-half_size:,half_size].reshape(-1,1)
    Z[-half_size:,:half_size] = band - np.abs( np.fliplr(Z[-half_size:, half_size+1:2*half_size+1]) - band )

    # solve system and convolve
    if derivative == None:
        m = np.linalg.pinv(A)[0].reshape((window_size, -1))
        return fftconvolve(Z, m, mode='valid')
    elif derivative == 'col':
        c = np.linalg.pinv(A)[1].reshape((window_size, -1))
        return fftconvolve(Z, -c, mode='valid')
    elif derivative == 'row':
        r = np.linalg.pinv(A)[2].reshape((window_size, -1))
        return fftconvolve(Z, -r, mode='valid')
    elif derivative == 'both':
        c = np.linalg.pinv(A)[1].reshape((window_size, -1))
        r = np.linalg.pinv(A)[2].reshape((window_size, -1))
        return fftconvolve(Z, -r, mode='valid'), fftconvolve(Z, -c, mode='valid')

# following code written by Daniel Buscombe, 1 May 2010
# updated 17 Oct 2010 to 
# 1) remove dependencies on numeric operations provided by matplotlib which have been superceded by numpy
# 2) implement trend surface filtering 
 
link = lambda a,b: concatenate((a,b[1:]))
edge = lambda a,b: concatenate(([a],[b]))
 
def qhull(sample):
     def dome(sample,base): 
         h, t = base
         dists = dot(sample-h, dot(((0,-1),(1,0)),(t-h)))
         outer = repeat(sample, dists>0, 0)
 
         if len(outer):
             pivot = sample[argmax(dists)]
             return link(dome(outer, edge(h, pivot)),
                         dome(outer, edge(pivot, t)))
         else:
             return base
 
     if len(sample) > 2:
     	axis = sample[:,0]
     	base = take(sample, [argmin(axis), argmax(axis)], 0)
     	return link(dome(sample, base),
                     dome(sample, base[::-1]))
     else:
 	return sample


def iseven(n):
   """Return true if n is even."""
   return n%2==0

def isodd(n):
   """Return true if n is odd."""   
   return not iseven(n)

def ascol( arr ):
    '''
    If the dimensionality of 'arr' is 1, reshapes it to be a column matrix (N,1).
    '''
    if len( arr.shape ) == 1: arr = arr.reshape( ( arr.shape[0], 1 ) )
    return arr
def asrow( arr ):
    '''
    If the dimensionality of 'arr' is 1, reshapes it to be a row matrix (1,N).
    '''
    if len( arr.shape ) == 1: arr = arr.reshape( ( 1, arr.shape[0] ) )
    return arr

def getP( auton, R ):

   #indg = nonzero(auton.flat >= R) # old version
   indg = (auton.flat >= R).nonzero()
   indg = np.array(indg)

   i = indg//auton.shape[1]
   j = indg % auton.shape[1]
   # To reduce the computation time, work with the boundary points only
   i=i[newaxis,:] # make column
   j=j[newaxis,:] # make column
   x=ind[i]
   y=ind[j]
   P=concatenate((x,y)) # stack columns i j next to each other
   P=P.squeeze() #remove singleton dimension
   P=P.T # transpose

   return P

############################################
# start main program
############################################
# deal with input arguments
#r=  image resolution mm/px
if len(sys.argv)>2:r=sys.argv[2] # looks for 'r', and assigns
else:r=1 # if r not found, r=1 mm/pix
# if not numeric, make numeric
if ~isinstance(r, (int,long,float,complex)):r=float(r)

if len(sys.argv)>3:dofilt=sys.argv[3] # looks for 'dofilt', and assigns
else:dofilt=1 # if dofilt not found, say yes
if ~isinstance(dofilt, (int,long,float,complex)):dofilt=float(dofilt)

# load image, convert to greyscale
print 'reading image ',sys.argv[1]
im=Image.open(sys.argv[1]).convert("L")

# info to be written away
imname=sys.argv[1] # the name of the input image
fname=sys.argv[0] # the name of this script
in1=str(r)

#import os
#fol=os.getcwd() # get current working directory
#title=fol + "/" + imname + "_out.txt" # make filename out of input image filename
title=imname + "_out.txt"
fout=open(title,"w") # (create and) open file
# print header info
fout.write("% grain size results ..."+"\n")
fout.write("% "+fname+"\n")
fout.write("% input image name ..."+"\n")
fout.write("% "+imname+"\n")
fout.write("% resolution:\n")
fout.write(in1+"\n")

nx, ny = im.size # find smallest dimension
mn=min(nx,ny)
print 'cropping image to ',mn,' by ',mn,' pixels'
box=(0,0,mn,mn) # make a box for cropping
region=im.crop(box) # crop to smallest dimension

if dofilt==1:
   # filtering - implemented Oct 2010
   Z = np.array(region)
   if isodd(mn/4):
      window_size=(mn/4)
   else:
      window_size=(mn/4)-1


   print 'filtering using window size ',window_size,' ... '
   # find low wavelength 2D linear polynomial
   Zf = sgolay2d( Z, window_size, order=8)
   # rescale filtered image to full range
   region=toimage(Z-Zf) #,cmin=Z.min(), cmax=Z.max())
   #region.save(imname+"_used.jpg")

imMean=mean(asarray(region)) # find mean
# 2D fourier transform of demeaned image
print 'calculating ... '
ft1=fft2(region-imMean)
mag1=pow(abs(ft1),2) # power spectrum

# autocovariance as inverse fourier transform as zero-centred power spectrum
auto=fftshift(real(ifft2(mag1)))
autoarray=asarray(auto) # make 1d to find max
ma=autoarray.max() # find max
auton=dot(auto,pow(ma,-1))

R=0.5
############################################
# the 1D mean size
#indg = nonzero(auton.flat >= R)
indg = (auton.flat >= R).nonzero()
indg = np.array(indg)

i = indg//auton.shape[1]
j = indg % auton.shape[1]
# centred coordinates because of zero-frequency shift
index=linspace(-(mn/2)+1,mn/2,mn)
xind=index[i]
yind=index[j]
# find elements >R+0.01
#indg = nonzero(auton.flat > R+0.01) # old version
indg = (auton.flat > R+0.01).nonzero()
indg = np.array(indg)

i = indg//auton.shape[1]
j = indg % auton.shape[1]
# remove i,j
auton[i,j]=nan;
# find elements >R-0.01 again, and retain i,j in index again
#indg = nonzero(auton.flat > R-0.01) # old version
indg = (auton.flat > R-0.01).nonzero()
indg = np.array(indg)

i = indg//auton.shape[1]
j = indg % auton.shape[1]  # n.b % is the modulus operator
xind=index[i]
yind=index[j]
modxind=abs(xind) # make non-symmetrical
modyind=abs(yind)
# find elements > mean(x)+5std(x) and mean(y)+5std(y) 
sx=modxind.std(axis=0)
sy=modyind.std(axis=0)
mx=modxind.mean(axis=0)
my=modyind.mean(axis=0)
#indx = nonzero(modxind.flat >5*sx) # old versions
#indy = nonzero(modyind.flat >5*sy)

indx = (modxind.flat >5*sx).nonzero()
indy = (modyind.flat >5*sy).nonzero()

# no longer needs to remove these elements
#xind[indx]=nan # remove those elements
#yind[indx]=nan
#xind[indy]=nan
#yind[indy]=nan
# find 'mean' lag from [x,y] coords
dx=0-pow(xind,2)
dy=0-pow(yind,2)
dxy=sqrt(abs(dx+dy)) # euclidean distance
m=nansum(dxy, axis=None)/size(dxy, axis=None) # mean distance
smean=(2*math.pi)/(pow(ceil(m),-1))*r # 2pi/k^-1r
############################################


############################################
# the 2D ellipse sizes
# find elements >R

nx, ny = region.size # find smallest dimension
k=min(nx,ny)

ind=arange(-(k/2)+1,k/2) # centre portion
ind=ind[newaxis,:] # make column
ind=ind.T

P=getP( auton, R )
K = qhull(P) # get convex hull

# need 6 points for the ellipse fit
if K.shape[1] >= 6:
   v=fitellipse(K, 'linear') # fit ellipse to boundary points
   s1=(2*math.pi)/(pow(ceil(v[1]),-1))*r # 2pi/k^-1r
   s2=(2*math.pi)/(pow(ceil(v[2]),-1))*r # 2pi/k^-1r

   gs=[smean, s1, s2] # make list
   gs=asarray(gs) # turn into array

   # print out most important result
   print "------------------------------------------------------------------"
   print "mean grain size (length units the same as input resolution) = ",gs.min()
   print "------------------------------------------------------------------"

   meanout=str(gs.min()) # for output file
   grainsout=str(gs) # for output file

else:

   Ksize = K.shape[0]
   #print "cannot fit ellipse to contour of R: only ",Ksize," boundary points"

   while Ksize < 6:
     R=R-0.01
     print "trying contour: ",R
     P=getP( auton, R )
     K = qhull(P) # get convex hull 
     Ksize = K.shape[0]
   
   v=fitellipse(K, 'linear') # fit ellipse to boundary points

   s1=(2*math.pi)/(pow(ceil(v[1]),-1))*r # 2pi/k^-1r
   s2=(2*math.pi)/(pow(ceil(v[2]),-1))*r # 2pi/k^-1r

   # implemented May 2011 upon comparison with matlab outputs
   slope=1.0071
   intercept=-3.5739
   s1=(slope*s1)+intercept
   s2=(slope*s2)+intercept
   gs=[s1, s2] # make list
   gs=asarray(gs) # turn into array

   # print out most important result
   print "------------------------------------------------------------------"
   print "mean grain size (length units the same as input resolution) = ",gs.min()
   print "------------------------------------------------------------------"

   meanout=str(gs.min()) # for output file
   grainsout=str(gs) # for output file


############################################
thetaout=str(ceil(v[3])/(math.pi/180)) # for output file

fout.write('% mean grain size:'+"\n")
fout.write(meanout+"\n")
fout.write('% grain sizes associated with mean, min, and max ellipse:'+"\n")
fout.write(grainsout+"\n")
fout.write('% preferred orientation of grains (degrees):'+"\n")
fout.write(thetaout+"\n")
fout.close()
print 'results saved to ',title
print "--------------------------------------"


